import axios from 'axios';
import {defineStore} from 'pinia';
import calculateGeoDistance from '../lib/calculateGeoDistance';
import type {FreeformErrors, Retailer} from './types';

type ContactType = 'user' | 'retailer' | 'installer';
type EnquiryType = 'sales' | 'technical' | 'other';

interface State {
    view: 'form' | 'success';
    retailers: Retailer[];
    locationCoords: { lat: number, lng: number } | null;
    formName: string | null;
    formEmail: string | null;
    formLocation: google.maps.places.AutocompletePrediction | null;
    formContactType: ContactType | null;
    formEnquiryType: EnquiryType | null;
    formRetailers: Retailer[];
    formMessage: string | null;
    errorName: string | null;
    errorEmail: string | null;
    errorLocation: string | null;
    errorContactType: string | null;
    errorEnquiryType: string | null;
    errorRetailers: string | null;
    errorMessage: string | null;
}

export const useStore = defineStore('global', {
    state: () => ({
        view: 'form',
        retailers: [],
        locationCoords: null,
        formName: null,
        formEmail: null,
        formLocation: null,
        formContactType: null,
        formEnquiryType: null,
        formRetailers: [],
        formMessage: null,
        errorName: null,
        errorEmail: null,
        errorLocation: null,
        errorContactType: null,
        errorEnquiryType: null,
        errorRetailers: null,
        errorMessage: null,
    } as State),
    getters: {
        hasErrors(state): boolean {
            return state.errorName !== null
                   || state.errorEmail !== null
                   || state.errorLocation !== null
                   || state.errorContactType !== null
                   || state.errorEnquiryType !== null
                   || state.errorRetailers !== null
                   || state.errorMessage !== null;
        },
        visibleRetailers(state): Retailer[] {
            if (state.locationCoords === null || state.formEnquiryType !== 'sales') {
                return [];
            }

            const retailers = state.retailers.map(r => ({
                retailer: r,
                distance: calculateGeoDistance({lat: r.latitude, lng: r.longitude}, state.locationCoords!),
            }));

            retailers.sort((a, b) => a.distance - b.distance);

            return retailers.slice(0, 4).map(({retailer}) => retailer);
        },
    },
    actions: {
        async fetchRetailers() {
            const response = await axios.get('/where-to-buy/retailers');

            if (response.status === 200) {
                this.retailers = response.data;
            }
        },
        resetErrors() {
            this.errorName = null;
            this.errorEmail = null;
            this.errorLocation = null;
            this.errorContactType = null;
            this.errorEnquiryType = null;
            this.errorRetailers = null;
            this.errorMessage = null;
        },
        validate() {
            this.resetErrors();

            if (this.formName == null || this.formName.trim() === '') {
                this.errorName = 'This field is required.';
            }
            if (this.formEmail == null || this.formEmail.trim() === '') {
                this.errorEmail = 'This field is required.';
            }
            if (this.formLocation == null) {
                this.errorLocation = 'This field is required.';
            }
            if (this.formContactType == null || this.formContactType.trim() === '') {
                this.errorContactType = 'This field is required.';
            }
            if (this.formEnquiryType == null || this.formEnquiryType.trim() === '') {
                this.errorEnquiryType = 'This field is required.';
            }
            if (this.formLocation != null && this.formEnquiryType === 'sales' && this.formRetailers.length === 0) {
                this.errorRetailers = 'This field is required.';
            }
            if (this.formMessage == null || this.formMessage.trim() === '') {
                this.errorMessage = 'This field is required.';
            }

            return this.hasErrors;
        },
        setResponseErrors(errors: FreeformErrors) {
            this.resetErrors();

            if (errors['name']?.length) {
                this.errorName = errors['name'][0];
            }
            if (errors['email']?.length) {
                this.errorEmail = errors['email'][0];
            }
            if (errors['location']?.length) {
                this.errorLocation = errors['location'][0];
            }
            if (errors['type_of_contact']?.length) {
                this.errorContactType = errors['type_of_contact'][0];
            }
            if (errors['type_of_enquiry']?.length) {
                this.errorEnquiryType = errors['type_of_enquiry'][0];
            }
            if (errors['retailers']?.length) {
                this.errorRetailers = errors['retailers'][0];
            }
            if (errors['message']?.length) {
                this.errorMessage = errors['message'][0];
            }
        },
    },
});
