import {unrefElement, useWindowScroll, useWindowSize} from '@vueuse/core';
import {defineStore} from 'pinia';
import {computed, reactive, ref, watch} from 'vue';
import {Background, BreakpointId, PinId, PinPosition} from './types';

const FeatureText = window.DifferenceBanner;
const BackgroundData: Record<BreakpointId, Background> = {
    [BreakpointId.xl]: {
        image: '/assets/images/difference-xl.jpg',
        width: 3978,
        height: 1473,
    },
    [BreakpointId.lg]: {
        image: '/assets/images/difference-lg.jpg',
        width: 2400,
        height: 1226,
    },
    [BreakpointId.md]: {
        image: '/assets/images/difference-md.jpg',
        width: 1575,
        height: 1425,
    },
};
const PinPositions: Record<PinId, Record<BreakpointId, PinPosition>> = {
    [PinId.TopLeft]: {
        [BreakpointId.xl]: {x: 1804, y: 704},
        [BreakpointId.lg]: {x: 1015, y: 613},
        [BreakpointId.md]: {x: 420, y: 918},
    },
    [PinId.TopMiddle]: {
        [BreakpointId.xl]: {x: 2306, y: 681},
        [BreakpointId.lg]: {x: 1445, y: 592},
        [BreakpointId.md]: {x: 746, y: 901},
    },
    [PinId.TopRight]: {
        [BreakpointId.xl]: {x: 2915, y: 781},
        [BreakpointId.lg]: {x: 1967, y: 674},
        [BreakpointId.md]: {x: 1138, y: 964},
    },
    [PinId.BottomLeft]: {
        [BreakpointId.xl]: {x: 1800, y: 1136},
        [BreakpointId.lg]: {x: 1011, y: 983},
        [BreakpointId.md]: {x: 418, y: 1197},
    },
    [PinId.BottomMiddle]: {
        [BreakpointId.xl]: {x: 2306, y: 906},
        [BreakpointId.lg]: {x: 1445, y: 786},
        [BreakpointId.md]: {x: 746, y: 1049},
    },
    [PinId.BottomRight]: {
        [BreakpointId.xl]: {x: 2915, y: 1058},
        [BreakpointId.lg]: {x: 1967, y: 920},
        [BreakpointId.md]: {x: 1138, y: 1148},
    },
};

export const useStore = defineStore('global', () => {
    const {width} = useWindowSize();
    const scroll = useWindowScroll();
    const breakpoint = computed(() => {
        if (width.value >= 1980) {
            return BreakpointId.xl;
        }
        if (width.value >= 1200) {
            return BreakpointId.lg;
        }
        return BreakpointId.md;
    });

    const modalPanel = ref<PinId | undefined>();
    const modalFeature = computed(() => modalPanel.value ? FeatureText[modalPanel.value] : undefined);
    const setModalPanel = (id?: PinId) => modalPanel.value = id;

    const pinRefs = reactive<Record<PinId, SVGGElement | undefined>>({});
    const pinHovers = reactive<Record<PinId, boolean | undefined>>({});
    const pinGlobalPositions = reactive<Record<PinId, DOMRect | undefined>>({});

    const background = computed(() => BackgroundData[breakpoint.value]);

    const setPinRef = (id: PinId) => (ref: SVGGElement) => pinRefs[id] = ref;
    const setPinHover = (id: PinId, state: boolean) => pinHovers[id] = state;

    const pinHover = (id: PinId) => pinHovers[id];
    const pinLabel = (id: PinId) => FeatureText[id].title;
    const pinLocalPosition = (id: PinId) => PinPositions[id][breakpoint.value];
    const pinGlobalPosition = (id: PinId) => pinGlobalPositions[id];

    watch([width, scroll.x, scroll.y, pinRefs], () => {
        for (const id of Object.values(PinId)) {
            pinGlobalPositions[id] = unrefElement(pinRefs[id])?.getBoundingClientRect();
        }
    }, {immediate: true});

    return {
        background,
        modalPanel,
        modalFeature,
        setModalPanel,
        setPinRef,
        setPinHover,
        pinLabel,
        pinHover,
        pinLocalPosition,
        pinGlobalPosition,
    };
});
