import {defineStore} from 'pinia';
import calculateGeoDistance from '../lib/calculateGeoDistance';
import stableSort from '../lib/stableSort';
import type {Retailer, SearchResult} from './types';

type View = 'search' | 'results' | 'error';
type ResultsView = 'combined' | 'list' | 'map';
type SearchRetailer = '' | 'all' | 'sales' | 'repairs';

interface State {
    view: View;
    resultsView: ResultsView;
    retailers: Retailer[];
    searchLocation?: google.maps.places.AutocompletePrediction;
    searchLocationError?: string;
    searchDistance: string;
    searchDistanceError?: string;
    searchRetailer: SearchRetailer;
    mapZoom?: number;
    mapCenter?: google.maps.LatLngLiteral;
    mapBounds?: google.maps.LatLngBounds;
    searchCenter?: google.maps.LatLngLiteral;
    selectedId?: number;
    emailRetailer?: Retailer;
}

export const useStore = defineStore('global', {
    state: () => ({
        view: 'search',
        resultsView: 'combined',
        retailers: [],
        searchLocation: undefined,
        searchLocationError: undefined,
        searchDistance: '',
        searchDistanceError: undefined,
        searchRetailer: '',
        mapZoom: undefined,
        mapCenter: undefined,
        mapBounds: undefined,
        searchCenter: undefined,
        selectedId: undefined,
        emailRetailer: undefined,
    } as State),
    getters: {
        filteredRetailers(state): Retailer[] {
            return state.retailers.filter(r => {
                switch (state.searchRetailer) {
                    case '':
                    case 'all':
                        return true;
                    case 'sales':
                        return r.services.includes('New Sales');
                    case 'repairs':
                        return r.services.includes('Repairs');
                }
            });
        },
        sortedRetailers(state): SearchResult[] {
            const center = {lat: state.searchCenter?.lat ?? 0, lng: state.searchCenter?.lng ?? 0};
            const getDistance = (r: Retailer) => calculateGeoDistance(center, {lat: r.latitude, lng: r.longitude});

            const withDistance = this.filteredRetailers.map((r) => ({r, d: getDistance(r)}));
            const sorted = stableSort(withDistance, (a, b) => a.d - b.d);

            return sorted.map(({r, d}, i) => ({
                retailer: r,
                distance: d,
                counter: `${i + 1}`,
                selected: r.id === state.selectedId,
            }));
        },
        visibleRetailers(state): SearchResult[] {
            return this.sortedRetailers.filter(r => state.mapBounds?.contains({lat: r.retailer.latitude, lng: r.retailer.longitude}));
        },
        emailOtherRetailers(state): Retailer[] {
            if (state.emailRetailer === undefined) {
                return [];
            }

            const retailers = state.retailers
                .filter(r => state.emailRetailer!.id !== r.id)
                .map(r => ({
                    retailer: r,
                    distance: calculateGeoDistance({lat: r.latitude, lng: r.longitude}, {lat: state.emailRetailer!.latitude, lng: state.emailRetailer!.longitude}),
                }));

            retailers.sort((a, b) => a.distance - b.distance);

            return retailers.slice(0, 4).map(({retailer}) => retailer);
        },
    },
    actions: {
        setSelected(retailerId?: number) {
            this.selectedId = retailerId;
        },
        setMapZoom(zoom: number) {
            this.mapZoom = zoom;
        },
        setMapCenter(point: google.maps.LatLng) {
            this.mapCenter = {
                lat: point.lat(),
                lng: point.lng(),
            };
        },
        setSearchCenter(point: google.maps.LatLng) {
            this.setMapCenter(point);
            this.searchCenter = {
                lat: point.lat(),
                lng: point.lng(),
            };
        },
        setMapBounds(bounds: google.maps.LatLngBounds) {
            this.mapBounds = bounds;
        },
        setResultsView(view: ResultsView) {
            this.resultsView = view;
        },
        async fetchRetailers() {
            const response = await fetch('/where-to-buy/retailers');

            if (response.ok) {
                this.retailers = await response.json();
                this.retailers.sort((a, b) => {
                    if (a.latitude === b.latitude) {
                        return a.longitude - b.longitude;
                    }
                    return b.latitude - a.latitude;
                });
            }
        },
    },
});
