import serialize from 'dom-serializer';
import fs from 'fs';
import {parseDocument} from 'htmlparser2';
import {types} from 'sass';
import {optimize} from 'svgo';
import projectPath from '../lib/projectPath';

export default [
    {
        declaration: 'svg-symbol($name, $attrs: (), $width: null, $height: null)',
        implementation: svgSymbol,
    },
    {
        declaration: 'svg-url($svg)',
        implementation: svgUrl,
    },
];

const spritePath = projectPath(PATH_CONFIG.src, PATH_CONFIG.sprites.src);

function svgUrl(svg) {
    return new types.String(`url("data:image/svg+xml;charset=utf8,${encodeURIComponent(svg.getValue())}")`);
}

/**
 * @param {types.String} name
 * @param {types.Map}    attrs
 * @param {types.Number} width
 * @param {types.Number} height
 * @returns {String}
 */
function svgSymbol(name, attrs = [], width = null, height = null) {
    const id = name.getValue();
    const path = `${spritePath}/${id}.svg`;
    const buffer = fs.readFileSync(path);
    const svgDom = getSvgDom(buffer.toString());

    const contents = extractSvgContents(svgDom);
    const {width: vw, height: vh} = extractSvgSize(svgDom);
    const styles = attrs ? buildSvgStyleAttrs(attrs) : '';

    let sw = vw;
    if (width instanceof types.Number) {
        if (width.getUnit() && width.getUnit() !== 'px') {
            throw new Error('SVG sizes must not have a unit.');
        }
        sw = width.getValue();
    }

    let sh = vh;
    if (height instanceof types.Number) {
        if (height.getUnit() && width.getUnit() !== 'px') {
            throw new Error('SVG sizes must not have a unit.');
        }
        sh = height.getValue();
    } else if (width instanceof types.Number) {
        sh = sw;
    }

    const data = svgTemplate(id, contents, {width: vw, height: vh}, {width: sw, height: sh}, styles);
    const result = optimize(data, TASK_CONFIG.images.svgo).data;

    return svgUrl(new types.String(result));
}

/**
 * @param {string} contents
 * @returns {Node}
 */
function getSvgDom(contents) {
    let svgDom = parseDocument(contents, {xmlMode: true});

    if (svgDom && svgDom.type === 'root') {
        svgDom = svgDom.children;
    }

    let index = 0;
    if (svgDom && svgDom[index] && svgDom[index].type === 'directive') {
        index++;
    }

    if (svgDom && svgDom[index] && svgDom[index].name === 'svg') {
        return svgDom[index];
    }

    throw new Error('Could not parse contents to create SVG DOM.');
}

function extractSvgContents(svgDom) {
    const children = svgDom.children
        .filter((el) => el.type === 'tag' && el.name !== 'title');

    return serialize(children);
}

function extractSvgSize(svgDom) {
    try {
        const {viewBox} = svgDom.attribs;
        const [_, width, height] = /\d+(?:\.\d+)? \d+(?:\.\d+)? (\d+(?:\.\d+)?) (\d+(?:\.\d+)?)/.exec(viewBox);
        return {width, height};
    } catch (e) {
        return {width: 0, height: 0};
    }
}

/**
 * @param {types.Map} attrs
 * @returns {string}
 */
function buildSvgStyleAttrs(attrs) {
    const attrsObj = sassMapToObj(attrs);
    const attrsArr = [];

    Object.keys(attrsObj)
        .map((k) => `${k}="${attrsObj[k]}"`)
        .forEach((a) => attrsArr.push(a));

    return attrsArr.join(' ');
}

/**
 * @param {types.Map} map
 * @returns {null}
 */
function sassMapToObj(map) {
    const obj = Object.create(null);

    for (let i = 0, il = map.getLength(); i < il; i++) {
        const key = map.getKey(i).getValue();
        const val = map.getValue(i);
        obj[key] = sassMapToObjVal(val);
    }

    return obj;
}

function sassMapToObjVal(val) {
    if (val instanceof types.Color || (val.constructor && val.constructor.name && val.constructor.name === 'SassColor')) {
        const r = parseInt(val.getR(), 10);
        const g = parseInt(val.getG(), 10);
        const b = parseInt(val.getB(), 10);
        const a = parseFloat(val.getA());
        if (a > 0) {
            return `rgba(${r},${g},${b},${a})`;
        } else {
            return `rgb(${r},${g},${b})`;
        }
    }

    return val.getValue();
}

function svgTemplate(id, paths, viewbox, size, styles) {
    return `
        <svg xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" viewBox="0 0 ${viewbox.width} ${viewbox.height}" width="${size.width}" height="${size.height}">
            <defs><symbol id="${id}">${paths}</symbol></defs>
            <use xlink:href="#${id}" ${styles} />
        </svg>
    `;
}
