import gulp from 'gulp';
import TerserPlugin from 'terser-webpack-plugin';
import VueLoaderPlugin from 'vue-loader/dist/plugin';
import {DefinePlugin} from 'webpack';
import webpack from 'webpack';
import bundleLogger from '../lib/bundleLogger';
import handleError from '../lib/handleError';
import projectPath from '../lib/projectPath';
import {sync} from './browser-sync';

export default function webpackTask(done, watch = false) {
    const compiler = webpack(TASK_CONFIG.webpack);
    const webpackError = handleError('webpack');

    const beforeHook = watch ? 'watchRun' : 'beforeRun';
    compiler.hooks[beforeHook].tapAsync('GulpWebpack', function (compiler, callback) {
        bundleLogger.start(Object.keys(compiler.options.entry).map(x => `${x}.js`));
        callback();
    });

    compiler.hooks.afterEmit.tapAsync('GulpWebpack', function (compilation, callback) {
        const outnames = Object.keys(compilation.assets);
        bundleLogger.end(outnames);
        sync.reload(outnames.map(x => `${compilation.compiler.outputPath}/${x}`));
        callback();
    });

    const callback = function (err, stats) {
        if (err) {
            webpackError(err);
        }
        if (stats.hasErrors()) {
            webpackError(new Error(stats.compilation.errors.join('\n    ')));
        }

        done();
    };

    if (watch) {
        compiler.watch({}, callback);
        setTimeout(() => bundleLogger.watch('webpack'), 1);
    } else {
        compiler.run(callback);
    }
}

webpackTask.displayName = 'webpack';
webpackTask.description = 'Bundle scripts and imported assets.';
webpackTask.defaultConfig = () => ({
    mode: GULP_ENV.debug ? 'development' : 'production',
    target: 'web',
    devtool: GULP_ENV.debug ? 'eval-source-map' : false,
    context: projectPath(PATH_CONFIG.src, PATH_CONFIG.scripts.src),
    output: {
        filename: '[name].js',
        chunkFilename: '[name].js',
        path: projectPath(PATH_CONFIG.dest, PATH_CONFIG.scripts.dest),
        publicPath: `/${PATH_CONFIG.dest}/${PATH_CONFIG.scripts.dest}`,
    },
    optimization: {
        minimize: !GULP_ENV.debug,
        minimizer: [
            new TerserPlugin({
                extractComments: false,
                terserOptions: {
                    format: {
                        comments: false,
                    },
                },
            }),
        ],
    },
    module: {
        noParse: /^(vue|vue-router|vuex|vuex-router-sync)$/,
        rules: [
            {
                test: /\.vue$/,
                loader: 'vue-loader',
            },
            {
                test: /\.vue$/,
                resourceQuery: /type=style/,
                sideEffects: true,
            },
            {
                test: /\.js$/,
                use: 'babel-loader',
                exclude: /(node_modules)/,
            },
            {
                test: /\.ts$/,
                use: [
                    'babel-loader',
                    {
                        loader: 'ts-loader',
                        options: {
                            transpileOnly: true,
                            happyPackMode: false,
                            appendTsSuffixTo: [
                                '\\.vue$',
                            ],
                        },
                    },
                ],
                exclude: /(node_modules)/,
            },
            {
                test: /\.css$/,
                use: ['vue-style-loader', 'css-loader'],
            },
        ],
    },
    plugins: [
        new VueLoaderPlugin(),
        new DefinePlugin({
            '__VUE_OPTIONS_API__': 'true',
            '__VUE_PROD_DEVTOOLS__': 'false',
        }),
        new DefinePlugin({
            'process.env': {
                NODE_ENV: GULP_ENV.debug ? '"development"' : '"production"',
            },
        }),
    ],
    resolve: {
        alias: {
            vue$: 'vue/dist/vue.runtime.esm-bundler.js',
        },
        extensions: ['.js', '.ts', '.vue', '.json'],
    },
});
webpackTask.register = () => {
    gulp.task(webpackTask);
};
