<?php

use BoldMinded\Reel\Service\CurlRequest;
use BoldMinded\Reel\Service\Helper;
use Vimeo\Vimeo;

/**
 * ExpressionEngine Reel Module Class
 *
 * @package     ExpressionEngine
 * @subpackage  Modules
 * @category    Reel
 * @author      Brian Litzinger
 * @copyright   Copyright (c) 2017 - Brian Litzinger
 * @link        http://boldminded.com/add-ons/reel
 * @license
 *
 * Copyright (c) 2011, 2012. BoldMinded, LLC
 * All rights reserved.
 *
 * This source is commercial software. Use of this software requires a
 * site license for each domain it is used on. Use of this software or any
 * of its source code without express written permission in the form of
 * a purchased commercial or other license is prohibited.
 *
 * THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
 * KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
 * PARTICULAR PURPOSE.
 *
 * As part of the license agreement for this software, all modifications
 * to this source must be submitted to the original author for review and
 * possible inclusion in future releases. No compensation will be provided
 * for patches, although where possible we will attribute each contribution
 * in file revision notes. Submitting such modifications constitutes
 * assignment of copyright to the original author (Brian Litzinger and
 * BoldMinded, LLC) for such modifications. If you do not wish to assign
 * copyright to the original author, your license to  use and modify this
 * source is null and void. Use of this software constitutes your agreement
 * to this clause.
 */

class Reel
{
    public $settings;
    private $params;
    private $cache;
    private $service;
    private $helper;

    const CACHE_PREFIX = 'reel/';
    const CACHE_TTL = 3600; // 1 hour
    const DEFAULT_WIDTH = 360;
    const DEFAULT_HEIGHT = 240;
    const MAX_RESULTS = 25;
    const YOUTUBE_ORDER = 'relevance';
    const VIMEO_ORDER = 'date';
    const SORT_DIRECTION = 'desc';

    public function __construct()
    {
        // Create cache
        if (!isset(ee()->session->cache['reel'])) {
            ee()->session->cache['reel'] = array();
        }
        $this->cache =& ee()->session->cache['reel'];

        /** @var Helper helper */
        $this->helper = ee(Helper::NAME);
        $this->settings = $this->helper->getSettings(true);
    }

    private function _get_params()
    {
        $this->params['settings']   = $this->settings[$this->service];
        $this->params['username']   = ee()->TMPL->fetch_param('user');
        $this->params['prefix']     = ee()->TMPL->fetch_param('variable_prefix', '');
        $this->params['width']      = ee()->TMPL->fetch_param('width', self::DEFAULT_WIDTH);
        $this->params['height']     = ee()->TMPL->fetch_param('height', self::DEFAULT_HEIGHT);
        $this->params['query']      = ee()->TMPL->fetch_param('query');
        $this->params['limit']      = ee()->TMPL->fetch_param('limit', self::MAX_RESULTS);
        $this->params['page']       = ee()->TMPL->fetch_param('page', '');
    }

    public function video()
    {
        $service = ee()->TMPL->fetch_param('service');

        if (!in_array($service, ['youtube', 'vimeo'])) {
            show_error(sprintf('The requested service <b>%s</b> is not supported.', $service));
        }

        return $this->$service();
    }

    /**
     * The template module tag for displaying a list of videos from YouTube based on a query.
     *
     * {exp:reel:youtube}
     *      {video_id}
     *      {title}
     *      {content} or {description}
     *      {url}
     *      {url_mobile}
     *      {thumbnail_0} {thumbnail_1} {thumbnail_2} {thumbnail_3}
     *      {author}
     *      {category}
     *      {keywords}
     *          {keyword}
     *      {/keywords}
     *      {duration}
     *      {view_count}
     *      {favorite_count}
     *      {id} - e.g. http://gdata.youtube.com/feeds/api/videos/[video id]
     *      {published}
     *      {updated}
     *      {embed}
     * {/exp:reel:youtube}
     */
    public function youtube()
    {
        $videos = [];
        $this->service = 'youtube';
        $this->_get_params();

        ee()->TMPL->log_item('Reel: Querying YouTube');

        $query_params = [
            'maxResults' => $this->params['limit'],
            'pageToken' => $this->params['page'],
            'q' => $this->params['query'],
        ];

        if (isset($this->params['username']) && $this->params['username'] != '') {
            $query_params['forUsername'] = $this->params['username'];
        }

        if (!isset($query_params['q'])) {
            ee()->TMPL->log_item('Reel: A query parameter is required to search YouTube.');
        }

        $prefix = $this->params['prefix'];
        $data = $this->load_youtube($query_params, true);

        if ( !$data || count($data['videos']) === 0) {
            ee()->TMPL->log_item('Reel: No Results found.');
            return ee()->TMPL->no_results();
        }

        $k = 0;
        foreach($data['videos'] as $video) {
            foreach ($video as $index => $value) {
                $videos[$k][$prefix.$index] = $value;
            }

            // finally add the full embed code...
            $videos[$k][$prefix.'embed'] = '<iframe src="http://www.youtube.com/embed/'. $videos[$k][$prefix.'video_id'] .'?rel=0&wmode=opaque&hd=1" border="0" width="'. $this->params['width'] .'" height="'. $this->params['height'] .'"></iframe>';

            $videos[$k][$prefix.'next_page'] = $data['nextPage'];
            $videos[$k][$prefix.'prev_page'] = $data['prevPage'];
            $videos[$k][$prefix.'absolute_results'] = $data['totalResults'];
            $videos[$k][$prefix.'total_results'] = $this->params['limit'];

            $k++;
        }

        return ee()->TMPL->parse_variables(ee()->TMPL->tagdata, $videos);
    }

    /**
     * @param array $params
     * @param bool $return Return as JSON (default)? If true will return PHP array instead.
     * @return array
     */
    public function load_youtube(array $params = [], $return = false)
    {
        if (!isset($this->settings['youtube']['api_key']) || $this->settings['youtube']['api_key'] == '') {
            show_error('Please enter your Google API Key in the Reel module settings page to search YouTube.');
        }

        $apiKey = $this->settings['youtube']['api_key'];

        $client = new Google_Client();
        $client->setApplicationName("Reel");
        $client->setDeveloperKey($apiKey);

        $params = array_merge([
            'type' => 'video',
            'order' => ee()->input->get('order') ?: self::YOUTUBE_ORDER,
            'maxResults' => ee()->input->get('perPage') ?: self::MAX_RESULTS,
            'q' => ee()->input->get('q')
        ], $params);

        if (ee()->input->get('page')) {
            $params['pageToken'] = ee()->input->get('page');
        }

        if (ee()->input->get('user')) {
            $params['forUsername'] = ee()->input->get('user');
        }

        if (ee()->input->get('channel')) {
            $params['inChannel'] = ee()->input->get('channel');
        }

        $cache_key = self::CACHE_PREFIX.'youtube/'.md5(serialize($params));

        if (($cache_results = ee()->cache->get($cache_key)) !== false) {
            if ($return) {
                return $cache_results;
            } else {
                $this->send_json_response($cache_results);
            }
        }

        // We only added it above to create a cache key. Its not a valid listSearch param.
        unset($params['forUsername']);
        unset($params['inChannel']);

        $youTubeService = new Google_Service_YouTube($client);

        $channelIds = [];

        // To search by username, we actually need to find all the channels they own
        if (ee()->input->get('user')) {
            $channelResults = $youTubeService->channels->listChannels('id,snippet', ['forUsername' => ee()->input->get('user')]);

            foreach ($channelResults->getItems() as $channel) {
                $channelIds[] = $channel->id;
            }
        }

        if (ee()->input->get('channel')) {
            $channelIds = explode(',', ee()->input->get('channel'));
        }

        if (ee()->input->get('user') && empty($channelIds)) {
            $this->send_json_response([
                'error' => 'No valid channels found.'
            ]);
        }

        if (!empty($channelIds)) {
            $params['channelId'] = implode(',', $channelIds);
        }

        $searchResults = $youTubeService->search->listSearch('id,snippet', $params);

        $response = [
            'nextPage' => $searchResults->nextPageToken,
            'prevPage' => $searchResults->prevPageToken,
            'totalResults' => $searchResults->getPageInfo()->getTotalResults(),
            'videos' => []
        ];

        $videoIds = [];
        $details = [];

        foreach ($searchResults as $item) {
            $videoIds[] = $item->getId()->getVideoId();
        }

        $parts = 'snippet,contentDetails,statistics,status';
        // ,fileDetails,statistics,processingDetails,recordingDetails,topicDetails
        $videoListResults = $youTubeService->videos->listVideos($parts, array(
            'id' => implode(',', $videoIds)
        ));

        /* @var Google_Service_YouTube_Video $video */
        foreach ($videoListResults as $video) {
            /* @var Google_Service_YouTube_VideoContentDetails $content */
            $content = $video->getContentDetails();
            /* @var Google_Service_YouTube_VideoStatistics $statistics */
            $statistics = $video->getStatistics();
            /* @var Google_Service_YouTube_VideoStatus $status */
            $status = $video->getStatus();
            /* @var Google_Service_YouTube_VideoSnippet $snippet */
            $snippet = $video->getSnippet();
            /* @var Google_Service_YouTube_ThumbnailDetails $thumbnails */
            $thumbnails = $snippet->getThumbnails();

            $length = new DateInterval($content->getDuration());
            $length = $length->format('%H:%I:%S');

            $defaultThumbnail = $thumbnails->getDefault();
            $mediumThumbnail = $thumbnails->getMedium();
            $highThumbnail = $thumbnails->getHigh();
            $standardThumbnail = $thumbnails->getStandard();
            $maxThumbnail = $thumbnails->getMaxres();

            $details[$video->getId()] = [
                'type' => 'youtube',
                'title' => $this->escape($snippet->getTitle()),
                'description' => $this->escape($snippet->getDescription()),
                'content' => $this->escape($snippet->getDescription()),
                'author' => $this->escape($snippet->getChannelTitle()),
                'duration' => $content->getDuration(),
                'length' => $length,
                'length_formatted' => $length,
                'channel_id' => $snippet->getChannelId(),
                'dimension' => $content->getDimension(),
                'licensed_content' => $content->getLicensedContent(),
                'caption' => $content->getCaption(),
                'definition' => $content->getDefinition(),
                'category' => $snippet->getCategoryId(),
                'published' => $snippet->getPublishedAt(),
                'thumbnail_default' => isset($defaultThumbnail->url) ? $defaultThumbnail->url : null,
                'thumbnail_medium' => isset($mediumThumbnail->url) ? $mediumThumbnail->url : null,
                'thumbnail_high' => isset($highThumbnail->url) ? $highThumbnail->url : null,
                'thumbnail_standard' => isset($standardThumbnail->url) ? $standardThumbnail->url : null,
                'thumbnail_max' => isset($maxThumbnail->url) ? $maxThumbnail->url : null,

                // Stats is null when not searching by a username
                'view_count' => $statistics ? $statistics->getViewCount() : null,
                'favorite_count' => $statistics ? $statistics->getFavoriteCount() : null,
                'comment_count' => $statistics ? $statistics->getCommentCount() : null,
                'dislike_count' => $statistics ? $statistics->getDislikeCount() : null,

                // Legacy variables
                'thumbnail_0' => isset($defaultThumbnail->url) ? $defaultThumbnail->url : null,
                'thumbnail_1' => isset($mediumThumbnail->url) ? $mediumThumbnail->url : null,
                'thumbnail_2' => isset($highThumbnail->url) ? $highThumbnail->url : null,
                'thumbnail_3' => isset($standardThumbnail->url) ? $standardThumbnail->url : null,
                'thumbnail_4' => isset($maxThumbnail->url) ? $maxThumbnail->url : null,
            ];
        }

        foreach ($searchResults as $video) {
            $videoId = $video->getId()->getVideoId();

            $row = array(
                'video_id' => $videoId,
                'url' => '//www.youtube.com/watch?v='.$videoId,
                'url_mobile' => '//m.youtube.com/watch?v='.$videoId,
            );

            if (isset($details[$videoId])) {
                $row = array_merge($row, $details[$videoId]);
            }

            $response['videos'][] = $row;
        }

        ee()->cache->save($cache_key, $response, $this->getCacheLifetime());

        if ($return) {
            return $response;
        }

        $this->send_json_response($response);

        return null;
    }

    /**
     * The template module tag for displaying a list of videos from Vimeo based on a query.
     *
     * {exp:reel:vimeo}
     *      {video_id}
     *      {title}
     *      {content} or {description}
     *      {url}
     *      {url_mobile}
     *      {thumbnail_0} {thumbnail_1} {thumbnail_2} {thumbnail_3}
     *      {author}
     *      {tags}
     *          {tag}
     *      {/tags}
     *      {duration}
     *      {view_count}
     *      {favorite_count}
     *      {id}
     *      {published}
     *      {updated}
     *      {embed}
     * {/exp:reel:vimeo}
     */
    public function vimeo()
    {
        $videos = [];
        $this->service = 'vimeo';
        $this->_get_params();
        $prefix = $this->params['prefix'];

        if (ee()->input->get('page')) {
            $this->params['page'] = ee()->input->get('page');
        }

        try
        {
            $method = 'videos.search';
            if ($this->params['username'] != '') {
                $method = 'videos.getAll';
            }

            $data = $this->load_vimeo($method, [
                'per_page'  => $this->params['limit'],
                'page'      => $this->params['page'],
                'query'     => $this->params['query'],
                'user'      => $this->params['username']
            ], true);

            if (isset($data['body']['error'])) {
                if (isset($data['body']['developer_message'])) {
                    show_error('Vimeo error: '. $data['body']['developer_message']);
                } else {
                    show_error('Vimeo error: '. $data['body']['error']);
                }
            }

            if (!$data || (isset($data['body']['data']) && count($data['body']['data']) === 0)) {
                return ee()->TMPL->no_results();
            }

            // Do we have a collection of videos, or just a single video?
            if (isset($data['body']['data'])) {
                $results = $data['body']['data'];
            } elseif (isset($data['body']['uri'])) {
                $data['body']['total'] = 1;
                $results = [$data['body']];
            }

            foreach ($results as $k => $video)
            {
                $videoId = str_replace('/videos/', '', $video['uri']);
                $videos[$k][$prefix.'title'] = $video['name'];

                // get basic info
                $videos[$k][$prefix.'id']               = $videoId;
                $videos[$k][$prefix.'video_id']         = $videoId;
                $videos[$k][$prefix.'content']          = $this->escape($video['description']);
                $videos[$k][$prefix.'description']      = $this->escape($video['description']);
                $videos[$k][$prefix.'published']        = isset($video['created_time']) ? strtotime($video['created_time']) : null;
                $videos[$k][$prefix.'updated']          = isset($video['modified_time']) ? strtotime($video['modified_time']) : null;
                $videos[$k][$prefix.'view_count']       = $video['stats']['plays'];
                //$videos[$k][$prefix.'favorite_count']   = $video->number_of_likes;
                $videos[$k][$prefix.'duration']         = $video['duration'];
                $videos[$k][$prefix.'author']           = $this->escape($video['user']['name']);
                $videos[$k][$prefix.'url_mobile']       = $video['link'];
                $videos[$k][$prefix.'url']              = $video['link'];

                foreach ($video['tags'] as $tag) {
                    $videos[$k][$prefix.'tags'][] = array($prefix.'tag' => $this->escape($tag['tag']));
                }

                // get video thumbnails
                foreach ($video['pictures']['sizes'] as $picIndex => $size) {
                    $videos[$k][$prefix.'thumbnail_'. $picIndex] = $size['link'];
                }

                // finally add the full embed code...
                $videos[$k][$prefix.'embed'] = '<iframe src="'. $this->helper->makeUrl('http://player.vimeo.com/video/') . $videos[$k][$prefix.'video_id'] .'?title=0&byline=0&portrait=0" frameborder="0" width="'. $this->params['width'] .'" height="'. $this->params['height'] .'" title="'. $this->escape($video['name']) .'"></iframe>';

                if ($this->params['page'] == '') {
                    $this->params['page'] = 1;
                }

                $videos[$k][$prefix.'next_page'] = (int) $this->params['page'] + 1;
                $videos[$k][$prefix.'prev_page'] =  $this->params['page'] <= 1 ? null : (int) $this->params['page'] - 1;
                $videos[$k][$prefix.'absolute_results'] = $data['body']['total'];
                $videos[$k][$prefix.'total_results'] = $this->params['limit'];
            }
        }
        catch (Exception $e)
        {
            ee()->TMPL->log_item('Reel: Could not load '. $this->params['url']);
        }

        return ee()->TMPL->parse_variables(ee()->TMPL->tagdata, $videos);
    }

    /**
     * @return int
     */
    private function getCacheLifetime()
    {
        if (isset($this->settings['global']['global_cache_lifetime']) && $this->settings['global']['global_cache_lifetime'] !== '') {
            return (int) $this->settings['global']['global_cache_lifetime'];
        }

        return self::CACHE_TTL;
    }

    /**
     * Search Vimeo, and return response.
     * Called only via an ACT request or from $this->vimeo()
     *
     * @param string $method
     * @param array $params
     * @param bool $return
     * @return array
     */
    public function load_vimeo(string $method = '', array $params = [], $return = false)
    {
        $method = $method ? $method : ee()->input->get('method');

        $data = null;

        $params['user'] = ee()->input->get('user') ?: (isset($params['user']) ? $params['user'] : null);
        $params['query'] = ee()->input->get('query') ?: (isset($params['query']) ? $params['query'] : null);
        $params['video_id'] = ee()->input->get('video_id') ?: (isset($params['video_id']) ? $params['video_id'] : null);
        $params['page'] = ee()->input->get('page') ?: ((isset($params['page']) && $params['page'] !== '') ? $params['page'] : 1);
        $params['per_page'] = ee()->input->get('perPage') ?: (isset($params['per_page']) ? $params['per_page'] : self::MAX_RESULTS);
        $params['sort'] = ee()->input->get('order') ?: (isset($params['order']) ? $params['order'] : self::VIMEO_ORDER);
        $params['direction'] = ee()->input->get('direction') ?: (isset($params['direction']) ? $params['direction'] : self::SORT_DIRECTION);
        $params['filter_playable'] = true;

        // Using JSON filters increases the API limit b/c we're not fetching the entire object.
        // https://developer.vimeo.com/api/common-formats#json-filter
        $params = array_merge([
            'fields' => 'uri,name,description,pictures,duration,stats,user,tags,link,created_time,modified_time'
        ], $params);

        ksort($params);

        $cacheKey = self::CACHE_PREFIX.'vimeo/'.$method.'/'.md5(serialize($params));

        if ($cached = ee()->cache->get($cacheKey)) {
            if ($return) {
                return $cached;
            }

            $this->send_json_response($cached);
        }

        $settings = $this->settings['vimeo'];

        $vimeo = new Vimeo(
            $settings['client_id'],
            $settings['client_secret'],
            $settings['client_token']
        );

        // Search by URL or video ID contributed by Colin Ambler & team at NXT Sports
        if (filter_var($params['query'], FILTER_VALIDATE_URL) || is_numeric($params['query'])) {
            if (is_numeric($params['query'])) {
                //the query matches as a number and we therefore we take it as video id
                $params['video_id'] = $params['query'];
            } else {
                //the query matches a url, we therefore have to split it and extract video id from the url
                $exploded = explode('/', $params['query']);
                $params['video_id'] = $exploded[3];
            }

            $method = 'videos.getInfo';
        }

        switch($method) {
            case "videos.search":
                $data = $vimeo->request('/videos', $params);
                break;

            case "videos.getInfo":
                $data = $vimeo->request('/videos/'.$params['video_id'], $params);
                break;

            default: // videos.getAll
                $data = $vimeo->request('/users/'. $params['user'] .'/videos', $params);
                break;
        }

        ee()->cache->save($cacheKey, $data, $this->getCacheLifetime());

        if ($return) {
            return $data;
        }

        $this->send_json_response($data);

        return null;
    }

    /**
     * @param $response
     * @return string
     */
    private function send_json_response($response)
    {
        ee()->output->enable_profiler(FALSE);
        @header('Content-Type: application/json; charset=UTF-8');
        echo json_encode($response);
        exit;
    }

    /**
     * @return bool
     */
    public function  isSecure() {
        return
            (!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] !== 'off')
            || $_SERVER['SERVER_PORT'] == 443;
    }

    /**
     * @param $string
     * @return string
     */
    private function escape($string)
    {
        return htmlentities($string, ENT_QUOTES);
    }

    private function authenticate(array $settings = [], $service = null)
    {
        $authEndpoint = Vimeo::AUTH_ENDPOINT;

        $curl = new CurlRequest($authEndpoint, [
            'grant_type' => 'client_credentials',
            'scope' => 'public',
        ], [
            'Authorization' => 'basic ' . base64_encode($settings['client_id'] . ':' . $settings['client_secret']),
            'Content-Type' => 'application/json',
            'Accept' => 'application/vnd.vimeo.*+json;version=3.4',
        ]);

        try {
            $response = $curl->post();
        } catch (\Exception $exception) {
            //throw $exception;
        }

        return $response;
    }
}
