<?php

use BoldMinded\Publisher\Service\Request;
use ExpressionEngine\Library\CP\EntryManager\ColumnInterface;
use Symfony\Component\Intl\Countries;
use Symfony\Component\Intl\Intl;

class Wm_address_ft extends \EE_Fieldtype implements ColumnInterface
{
    /** @var array */
    public $info = [
        'name'    => WM_ADDRESS_NAME,
        'version' => WM_ADDRESS_VERSION,
    ];

    /** @var array */
    public $content_types = [
        'channel',
        'grid',
    ];

    /** @var array */
    public $default_data = [
        'name'        => '',
        'street'      => '',
        'street_2'    => '',
        'city'        => '',
        'region'      => '',
        'postal_code' => '',
        'country'     => '',
    ];

    /** @var array */
    public $default_settings = [
        'display_name' => 'y',
    ];

    /** @var array */
    public $default_global_settings = [
        'display_name'  => 'y',
        'top_countries' => [],
    ];

    /** @var bool */
    public $has_array_data = true;

    public $entry_manager_compatible = true;

    /**
     * Wm_address_ft constructor.
     */
    public function __construct()
    {
        parent::__construct();

        ee()->lang->loadfile('wm_address');
    }

    /**
     * @return array
     */
    public function install()
    {
        return $this->default_global_settings;
    }

    /**
     * @param string $current
     * @return bool
     */
    public function update($current)
    {
        return true;
    }

    /**
     * @param string $name
     * @return bool
     */
    public function accepts_content_type($name)
    {
        return in_array($name, $this->content_types, true);
    }

    /**
     * @return array|string
     */
    public function display_global_settings()
    {
        $data = $this->get_settings_from_post(true);
        $settings = $this->merge_settings($data);

        return ee('View')->make('wm_address:display_global_settings')->render([
            'settings' => $this->build_global_settings($settings),
        ]);
    }

    /**
     * @return array
     */
    public function save_global_settings()
    {
        return $this->get_settings_from_post(true);
    }

    /**
     * @param array $data
     * @return string|array
     */
    public function display_settings($data)
    {
        $settings = $this->merge_settings($data);
        $settings = $this->build_settings($settings);

        if ($this->content_type() === 'grid') {
            return ['field_options' => $settings];
        }

        return [
            'field_options_wm_address' => [
                'label'    => 'field_options',
                'group'    => 'wm_address',
                'settings' => $settings,
            ],
        ];
    }

    /**
     * @param array $data
     * @return array
     */
    public function save_settings($data)
    {
        return array_intersect_key($data, $this->default_settings);
    }

    /**
     * @param array|string $data
     * @return array
     */
    public function pre_process($data)
    {
        $data = $this->merge_data($data);
        $data['country_code'] = $data['country'];

        /** @var \ExpressionEngine\Service\Addon\Addon|null $publisher */
        $publisher = ee('Addon')->get('publisher');

        if ($publisher && $publisher->isInstalled()) {
            /** @var \BoldMinded\Publisher\Model\Language $language */
            $language = ee(Request::NAME)->getCurrentLanguage();
            if ($language) {
                $locale = $language->getLocale();
                $country = strtoupper($data['country']);
                $data['country_name'] = Countries::getName($country, $locale);
            }
        }

        if (!isset($data['country_name'])) {
            $data['country_name'] = ee('wm_address:Countries')->get($data['country']);
        }

        return $data;
    }

    /**
     * @param array|string $data
     * @return string
     */
    public function display_field($data)
    {
        $this->include_css();

        $data = $this->merge_data($data);
        $settings = $this->merge_settings($data);
        $countries = ee('wm_address:Countries')->dropdownOptions($settings['top_countries']);

        if ($this->content_type() === 'grid') {
            $template = 'wm_address:grid_display_field';
        } else {
            $template = 'wm_address:display_field';
        }

        return ee('View')->make($template)->render([
            'data'       => $data,
            'settings'   => $settings,
            'countries'  => $countries,
            'field_name' => $this->field_name,
        ]);
    }

    /**
     * @param array|string $data
     * @return array|bool|string
     */
    public function validate($data)
    {
        $data = $this->merge_data($data);

        // If the field is required, we need at least a street address and city
        if ($this->is_field_required()) {
            if (empty($data['street']) || empty($data['city'])) {
                return lang('required');
            }
        }

        return true;
    }

    /**
     * @param array $data
     * @return string
     */
    public function save($data)
    {
        if (!is_array($data)) {
            $data = [];
        }

        $data = array_intersect_key($data, $this->default_data);
        $data = array_diff_assoc($data, $this->default_data);

        return $this->serialize_data($data);
    }

    /**
     * @param array $data
     * @return string
     */
    public function replace_is_empty($data)
    {
        return count(array_filter($data)) === 0 ? 'y' : '';
    }

    /**
     * @param array $data
     * @return string
     */
    public function replace_is_not_empty($data)
    {
        return count(array_filter($data)) > 0 ? 'y' : '';
    }

    /**
     * @param $data
     * @return string
     */
    public function replace_name($data)
    {
        return $data['name'];
    }

    /**
     * @param array $data
     * @return string
     */
    public function replace_street($data)
    {
        return $data['street'];
    }

    /**
     * @param array $data
     * @return string
     */
    public function replace_street_2($data)
    {
        return $data['street_2'];
    }

    /**
     * @param array $data
     * @return string
     */
    public function replace_city($data)
    {
        return $data['city'];
    }

    /**
     * @param array $data
     * @return string
     */
    public function replace_region($data)
    {
        return $data['region'];
    }

    /**
     * @param array $data
     * @return string
     */
    public function replace_postal_code($data)
    {
        return $data['postal_code'];
    }

    /**
     * @param array $data
     * @return string
     */
    public function replace_country($data)
    {
        return $data['country_name'];
    }

    /**
     * @param array $data
     * @return string
     */
    public function replace_country_code($data)
    {
        return $data['country_code'];
    }

    /**
     * @param array $data
     * @param array $params
     * @return string
     */
    public function replace_map_url($data, array $params = [])
    {
        $include_name = array_key_exists('include_name', $params) && $params['include_name'] === 'yes';
        $extra_params = array_key_exists('params', $params) ? '&' . strtolower($params['params']) : '';
        $source = array_key_exists('source', $params) ? strtolower($params['source']) : 'google';

        if (!$include_name && !empty($data['name'])) {
            $data['name'] = '';
        }

        if (!empty($params['name'])) {
            $data['name'] = $params['name'];
        }

        $query = urlencode($this->replace_inline($data));

        switch ($source) {
            case 'bing':
                return "http://www.bing.com/maps/?v=2&where1={$query}{$extra_params}";

            case 'mapquest':
                return "http://mapq.st/map?q={$query}{$extra_params}";

            case 'google':
            default:
                return "http://maps.google.com/maps?q={$query}{$extra_params}";
        }
    }

    /**
     * @param array $data
     * @param array $params
     * @return string
     */
    public function replace_static_map($data, array $params = [])
    {
        $width = array_key_exists('width', $params) ? $params['width'] : '400';
        $height = array_key_exists('height', $params) ? $params['height'] : '200';

        $map_url = $this->replace_static_map_url($data, $params);
        $address = $this->replace_inline($data);

        return sprintf('<img src="%s" alt="%s" width="%s" height="%s">', $map_url, $address, $width, $height);
    }

    /**
     * @param array $data
     * @param array $params
     * @return string
     */
    public function replace_static_map_url($data, array $params = [])
    {
        $include_name = array_key_exists('include_name', $params) && $params['include_name'] === 'yes';
        $zoom = array_key_exists('zoom', $params) ? $params['zoom'] : '14';
        $width = array_key_exists('width', $params) ? $params['width'] : '400';
        $height = array_key_exists('height', $params) ? $params['height'] : '200';
        $scale = array_key_exists('scale', $params) ? $params['scale'] : '1';
        $format = array_key_exists('format', $params) ? $params['format'] : 'png';
        $type = array_key_exists('type', $params) ? $params['type'] : 'roadmap';
        $marker_size = array_key_exists('marker:size', $params) ? $params['marker:size'] : false;
        $marker_label = array_key_exists('marker:label', $params) ? $params['marker:label'] : false;
        $marker_color = array_key_exists('marker:color', $params) ? str_replace('#', '0x', $params['marker:color']) : false;
        $api_key = array_key_exists('api_key', $params) ? $params['api_key'] : ee()->config->item('wm_address_gmaps_api_key');

        if (!$include_name && !empty($data['name'])) {
            $data['name'] = '';
        }

        $query = urlencode($this->replace_inline($data));

        $marker = '';
        $marker .= $marker_size ? 'size:' . $marker_size . '|' : '';
        $marker .= $marker_color ? 'color:' . $marker_color . '|' : '';
        $marker .= $marker_label ? 'label:' . $marker_label . '|' : '';

        $output = 'https://maps.googleapis.com/maps/api/staticmap';
        $output .= '?zoom=' . $zoom;
        $output .= '&size=' . $width . 'x' . $height;
        $output .= '&scale=' . $scale;
        $output .= '&format=' . $format;
        $output .= '&maptype=' . $type;
        $output .= '&markers=' . $marker . $query;

        if ($api_key) {
            $output .= '&key=' . $api_key;
        }

        return $output;
    }

    /**
     * @param array       $data
     * @param array       $params
     * @param string|bool $tagdata
     * @return string
     */
    public function replace_tag($data, $params = [], $tagdata = false)
    {
        if ($tagdata) {
            $data['country'] = $data['country_name'];

            return ee()->TMPL->parse_variables($tagdata, [$data]);
        }

        return $this->replace_plain($data, $params);
    }

    /**
     * @param array $data
     * @param array $params
     * @return string
     */
    public function replace_inline($data, array $params = [])
    {
        $output = $this->replace_plain($data, $params);
        $output = str_replace("\n", ', ', $output);

        return $output;
    }

    /**
     * @param array $data
     * @param array $params
     * @return string
     */
    public function replace_html($data, array $params = [])
    {
        $output = $this->replace_plain($data, $params);
        $output = str_replace("\n", '<br>', $output);

        return $output;
    }

    /**
     * @param array $data
     * @param array $params
     * @return string
     */
    public function replace_rdfa($data, array $params = [])
    {
        $output = "<div xmlns:v=\"http://rdf.data-vocabulary.org/#\" typeof=\"v:Address\" class=\"adr\">\n";

        if ($data['name']) {
            $output .= "    <div property=\"v:name\">{$data['name']}</div>\n";
        }
        if ($data['street'] || $data['street_2']) {
            $output .= "    <div property=\"v:street-address\">\n";
        }
        if ($data['street']) {
            $output .= "        <div property=\"v:street-address\">{$data['street']}</div>\n";
        }
        if ($data['street_2']) {
            $output .= "        <div property=\"v:extended-address\">{$data['street_2']}</div>\n";
        }
        if ($data['street'] || $data['street_2']) {
            $output .= "    </div>\n";
        }

        $output .= "    <div>\n";
        $output .= "        <span property=\"v:locality\" class=\"locality\">{$data['city']}</span>,\n";
        $output .= "        <span property=\"v:region\" class=\"region\">{$data['region']}</span>\n";
        $output .= "        <span property=\"v:postal-code\" class=\"postal-code\">{$data['postal_code']}</span>\n";
        $output .= "    </div>\n";
        if (!isset($params['display_country']) || $params['display_country'] !== 'no') {
            $output .= "    <div property=\"v:contry-name\" class=\"country\">{$data['country_name']}</div>\n";
        }
        $output .= '</div>';

        return $output;
    }

    /**
     * @param array $data
     * @param array $params
     * @return string
     */
    public function replace_schema($data, array $params = [])
    {
        $output = "<div itemprop=\"address\" itemscope itemtype=\"http://schema.org/PostalAddress\" class=\"adr\">\n";

        if ($data['name']) {
            $output .= "    <div itemprop=\"name\">{$data['name']}</div>\n";
        }
        if ($data['street'] || $data['street_2']) {
            $output .= "    <div itemprop=\"streetAddress\">\n";
        }
        if ($data['street']) {
            $output .= "        <div itemprop=\"street-address\">{$data['street']}</div>\n";
        }
        if ($data['street_2']) {
            $output .= "        <div itemprop=\"extended-address\">{$data['street_2']}</div>\n";
        }
        if ($data['street'] || $data['street_2']) {
            $output .= "    </div>\n";
        }

        $output .= "    <div>\n";
        $output .= "        <span itemprop=\"addressLocality\" class=\"locality\">{$data['city']}</span>,\n";
        $output .= "        <span itemprop=\"addressRegion\" class=\"region\">{$data['region']}</span>\n";
        $output .= "        <span itemprop=\"postalCode\" class=\"postal-code\">{$data['postal_code']}</span>\n";
        $output .= "    </div>\n";
        if (!isset($params['display_country']) || $params['display_country'] !== 'no') {
            $output .= "    <div itemprop=\"addressCountry\" class=\"country\">{$data['country_name']}</div>\n";
        }
        $output .= '</div>';

        return $output;
    }

    /**
     * @param array $data
     * @param array $params
     * @return string
     */
    public function replace_microformat($data, array $params = [])
    {
        $output = "<div class=\"adr\">\n";

        if ($data['street']) {
            $output .= "    <div class=\"street-address\">{$data['street']}</div>\n";
        }
        if ($data['street_2']) {
            $output .= "    <div class=\"extended-address\">{$data['street_2']}</div>\n";
        }

        $output .= "    <div>\n";
        $output .= "        <span class=\"locality\">{$data['city']}</span>,\n";
        $output .= "        <span class=\"region\">{$data['region']}</span>\n";
        $output .= "        <span class=\"postal-code\">{$data['postal_code']}</span>\n";
        $output .= "    </div>\n";
        if (!isset($params['display_country']) || $params['display_country'] !== 'no') {
            $output .= "    <div class=\"country\">{$data['country_name']}</div>\n";
        }
        $output .= '</div>';

        if ($data['name']) {
            $inner = $output;
            $output = "<div class=\"vcard\">\n";
            $output .= "    <div itemprop=\"name\">{$data['name']}</div>\n";
            $output .= '    ' . str_replace("\n", "\n    ", $inner) . "\n";
            $output .= '</div>';
        }

        return $output;
    }

    /**
     * @param array $data
     * @param array $params
     * @return string
     */
    private function replace_plain($data, array $params = [])
    {
        $output = [];

        if ($data['name']) {
            $output[] = $data['name'];
        }

        if ($data['street']) {
            $output[] = $data['street'];
        }

        if ($data['street_2']) {
            $output[] = $data['street_2'];
        }

        $output[] = trim("{$data['city']}, {$data['region']} {$data['postal_code']}", ' ,');

        if (!isset($params['display_country']) || $params['display_country'] !== 'no') {
            $output[] = $data['country_name'];
        }

        return implode("\n", array_filter($output));
    }

    /**
     * @param array|string $data
     * @return array
     */
    private function merge_data($data)
    {
        $data = $this->unserialize_data($data);
        $data = array_merge($this->default_data, $data);
        $data = array_intersect_key($data, $this->default_data);

        return $data;
    }

    /**
     * @param array $data
     * @return array
     */
    private function merge_settings(array $data)
    {
        $defaults = array_merge($this->default_global_settings, $this->default_settings);

        $settings = array_merge($defaults, $this->settings, $data);
        $settings = array_intersect_key($settings, $defaults);

        return $settings;
    }

    /**
     * @param array $settings
     * @return array
     */
    private function build_settings(array $settings)
    {
        return [
            [
                'title'  => 'wm_address_display_name',
                'desc'   => 'wm_address_display_name_desc',
                'fields' => [
                    'display_name' => [
                        'type'  => 'yes_no',
                        'value' => $settings['display_name'],
                    ],
                ],
            ],
        ];
    }

    /**
     * @param array $settings
     * @return array
     */
    private function build_global_settings(array $settings)
    {
        return array_merge($this->build_settings($settings), [
            [
                // TODO: This would be better with a filter like the grid ft
                'title'  => 'wm_address_top_countries',
                'desc'   => 'wm_address_top_countries_desc',
                'fields' => [
                    'top_countries' => [
                        'type'    => 'checkbox',
                        'choices' => ee('wm_address:Countries')->all(),
                        'wrap'    => true,
                        'value'   => $settings['top_countries'],
                    ],
                ],
            ],
        ]);
    }

    /**
     * @param array $data
     * @return string
     */
    private function serialize_data(array $data)
    {
        return json_encode($data);
    }

    /**
     * @param array|string $data
     * @return array
     */
    private function unserialize_data($data)
    {
        // Check if data is received as a serialized string
        if (is_string($data)) {
            $data = json_decode($data, true);
        }

        // Data won't decode to an array if it is an empty string
        if (!is_array($data)) {
            $data = [];
        }

        return $data;
    }

    /**
     * @return void
     */
    private function include_css()
    {
        if (ee()->session->cache(__CLASS__, 'css') !== true) {
            ee()->session->set_cache(__CLASS__, 'css', true);

            $css = file_get_contents(__DIR__ . '/css/wm_address.css');
            ee()->cp->add_to_head('<style type="text/css">' . $css . '</style>');
        }
    }

    /**
     * @return bool
     */
    private function is_field_required()
    {
        return array_key_exists('field_required', $this->settings) &&
               $this->settings['field_required'];
    }

    /**
     * @param bool $global
     * @return array
     */
    private function get_settings_from_post($global = false)
    {
        $settings = [];
        $defaults = $global ? $this->default_global_settings : $this->default_settings;

        foreach ($defaults as $key => $default) {
            $value = ee()->input->post($key);
            if ($value !== false) {
                $settings[$key] = $value ?: $default;
            }
        }

        return $settings;
    }

    public function renderTableCell($data, $field_id, $entry)
    {
        if ($data === null) {
            return '';
        }

        $data = $this->unserialize_data($data);
        return $this->replace_postal_code($data);
    }
}
